@@ -3,8 +3,20 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.7.3   2011/04/01
+ * Version: 1.8.1   2011/04/01
  *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License v2 as published by the
+ * Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 #include "include.h"
 
@@ -40,7 +52,7 @@ static void show_result2(int result)
 {
 	if (is_enforce) {
 		if (result == EOF) {
-			if (errno == ENOMEM)
+			if (errno == EAGAIN)
 				printf("OK: Permission denied.\n");
 			else
 				printf("FAILED: %s\n", strerror(errno));
@@ -56,29 +68,6 @@ static void show_result2(int result)
 }
 
 static void show_result3(int result)
-{
-	if (is_enforce) {
-		if (result == EOF) {
-			if (errno == ECONNABORTED)
-				printf("OK: Permission denied.\n");
-			else
-				printf("FAILED: %s\n", strerror(errno));
-		} else {
-			if (write(result, "", 1) == EOF && errno == EPERM)
-				printf("OK: Permission denied after accept()."
-				       "\n");
-			else
-				printf("BUG!(%d)\n", result);
-		}
-	} else {
-		if (result != EOF)
-			printf("OK\n");
-		else
-			printf("%s\n", strerror(errno));
-	}
-}
-
-static void show_result4(int result)
 {
 	if (result == EOF) {
 		if (errno == EDESTADDRREQ)
@@ -95,25 +84,20 @@ static void show_result4(int result)
 static void set_enforce(int flag)
 {
 	is_enforce = flag;
-	if (flag) {
-		set_profile(3, "network::inet_udp_bind");
-		set_profile(3, "network::inet_udp_connect");
-		set_profile(3, "network::inet_tcp_bind");
-		set_profile(3, "network::inet_tcp_listen");
-		set_profile(3, "network::inet_tcp_connect");
-		set_profile(3, "network::inet_tcp_accept");
-		set_profile(3, "network::inet_raw_bind");
-		set_profile(3, "network::inet_raw_connect");
-	} else {
-		set_profile(2, "network::inet_udp_bind");
-		set_profile(2, "network::inet_udp_connect");
-		set_profile(2, "network::inet_tcp_bind");
-		set_profile(2, "network::inet_tcp_listen");
-		set_profile(2, "network::inet_tcp_connect");
-		set_profile(2, "network::inet_tcp_accept");
-		set_profile(2, "network::inet_raw_bind");
-		set_profile(2, "network::inet_raw_connect");
-	}
+	if (flag)
+		flag = 3;
+	else
+		flag = 2;
+	set_profile(flag, "network::inet_stream_bind");
+	set_profile(flag, "network::inet_stream_listen");
+	set_profile(flag, "network::inet_stream_connect");
+	set_profile(flag, "network::inet_stream_accept");
+	set_profile(flag, "network::inet_dgram_bind");
+	set_profile(flag, "network::inet_dgram_send");
+	set_profile(flag, "network::inet_dgram_recv");
+	set_profile(flag, "network::inet_raw_bind");
+	set_profile(flag, "network::inet_raw_send");
+	set_profile(flag, "network::inet_raw_recv");
 }
 
 static void stage_network_test(void)
@@ -145,11 +129,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		getsockname(fd1, (struct sockaddr *) &saddr, &size);
 
 		snprintf(sbuffer, sizeof(sbuffer) - 1,
@@ -170,20 +154,22 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		getsockname(fd2, (struct sockaddr *) &caddr, &size);
 
 		snprintf(sbuffer, sizeof(sbuffer) - 1,
 			 "Server: Accepting TCP 127.0.0.1 %d",
 			 ntohs(caddr.sin_port));
 		set_enforce(1);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) | O_NONBLOCK);
 		show_prompt(sbuffer);
 		fd3 = accept(fd1, (struct sockaddr *) &caddr, &size);
-		show_result3(fd3);
+		show_result2(fd3);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) & ~O_NONBLOCK);
 
 		set_enforce(0);
 		close(fd2);
@@ -195,9 +181,11 @@ static void stage_network_test(void)
 			 "Server: Accepting TCP 127.0.0.1 %d",
 			 ntohs(caddr.sin_port));
 		set_enforce(0);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) | O_NONBLOCK);
 		show_prompt(sbuffer);
 		fd3 = accept(fd1, (struct sockaddr *) &caddr, &size);
-		show_result3(fd3);
+		show_result2(fd3);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) & ~O_NONBLOCK);
 
 		close(fd3);
 		close(fd2);
@@ -210,6 +198,7 @@ static void stage_network_test(void)
 		socklen_t size = sizeof(saddr);
 		int fd1 = EOF;
 		int fd2 = EOF;
+		int ret_ignored;
 
 		fd1 = socket(PF_INET, SOCK_DGRAM, 0);
 
@@ -222,11 +211,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		getsockname(fd1, (struct sockaddr *) &saddr, &size);
 
 		fd2 = socket(PF_INET, SOCK_DGRAM, 0);
@@ -238,10 +227,10 @@ static void stage_network_test(void)
 			 ntohs(saddr.sin_port));
 		set_enforce(1);
 		show_prompt(cbuffer);
-		show_result4(send(fd2, "", 1, 0));
+		show_result3(send(fd2, "", 1, 0));
 		set_enforce(0);
 		show_prompt(cbuffer);
-		show_result4(send(fd2, "", 1, 0));
+		show_result3(send(fd2, "", 1, 0));
 
 		/* write() -> read() */
 
@@ -250,10 +239,10 @@ static void stage_network_test(void)
 			 ntohs(saddr.sin_port));
 		set_enforce(1);
 		show_prompt(cbuffer);
-		show_result4(write(fd2, "", 1));
+		show_result3(write(fd2, "", 1));
 		set_enforce(0);
 		show_prompt(cbuffer);
-		show_result4(write(fd2, "", 1));
+		show_result3(write(fd2, "", 1));
 
 		/* connect() */
 
@@ -263,11 +252,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		getsockname(fd2, (struct sockaddr *) &caddr, &size);
 
 		/* sendto() -> recvfrom() */
@@ -282,22 +271,22 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
-				  sizeof(saddr)));
+				   sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
-				  sizeof(saddr)));
+				   sizeof(saddr)));
 		set_enforce(1);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 		set_enforce(0);
 		sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
 		       sizeof(saddr));
 		set_enforce(0);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 
 		/* send() -> recv() */
 
@@ -319,12 +308,12 @@ static void stage_network_test(void)
 		show_result(send(fd2, "", 1, 0));
 		set_enforce(1);
 		show_prompt(sbuffer);
-		show_result2(recv(fd1, buf, sizeof(buf) - 1, 0));
+		show_result2(recv(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT));
 		set_enforce(0);
 		send(fd2, "", 1, 0);
 		set_enforce(0);
 		show_prompt(sbuffer);
-		show_result2(recv(fd1, buf, sizeof(buf) - 1, 0));
+		show_result2(recv(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT));
 
 		/* write() -> read() */
 
@@ -345,10 +334,12 @@ static void stage_network_test(void)
 		show_prompt(cbuffer);
 		show_result(write(fd2, "", 1));
 		set_enforce(1);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) | O_NONBLOCK);
 		show_prompt(sbuffer);
 		show_result2(read(fd1, buf, sizeof(buf) - 1));
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) & ~O_NONBLOCK);
 		set_enforce(0);
-		write(fd2, "", 1);
+		ret_ignored = write(fd2, "", 1);
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result2(read(fd1, buf, sizeof(buf) - 1));
@@ -385,12 +376,12 @@ static void stage_network_test(void)
 			show_result(sendmsg(fd1, &msg1, 0));
 			set_enforce(1);
 			show_prompt(sbuffer);
-			show_result2(recvmsg(fd1, &msg2, 0));
+			show_result2(recvmsg(fd1, &msg2, MSG_DONTWAIT));
 			set_enforce(0);
 			sendmsg(fd1, &msg1, 0);
 			set_enforce(0);
 			show_prompt(sbuffer);
-			show_result2(recvmsg(fd1, &msg2, 0));
+			show_result2(recvmsg(fd1, &msg2, MSG_DONTWAIT));
 		}
 
 		close(fd2);
@@ -417,11 +408,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 
 		fd2 = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
 
@@ -432,11 +423,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 
 		memset(&ip, 0, sizeof(ip));
 		ip.version = 4;
@@ -451,11 +442,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, &ip, sizeof(ip), 0,
-				  (struct sockaddr *) &saddr, sizeof(saddr)));
+				   (struct sockaddr *) &saddr, sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, &ip, sizeof(ip), 0,
-				  (struct sockaddr *) &saddr, sizeof(saddr)));
+				   (struct sockaddr *) &saddr, sizeof(saddr)));
 
 		getsockname(fd2, (struct sockaddr *) &caddr, &size);
 		snprintf(sbuffer, sizeof(sbuffer) - 1,
@@ -463,15 +454,15 @@ static void stage_network_test(void)
 			 ntohs(caddr.sin_port));
 		set_enforce(1);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 		set_enforce(0);
 		sendto(fd2, &ip, sizeof(ip), 0, (struct sockaddr *) &saddr,
 		       sizeof(saddr));
 		set_enforce(0);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 
 		close(fd2);
 		close(fd1);
@@ -502,11 +493,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		getsockname(fd1, (struct sockaddr *) &saddr, &size);
 
 		snprintf(sbuffer, sizeof(sbuffer) - 1,
@@ -527,20 +518,22 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 
 		getsockname(fd2, (struct sockaddr *) &caddr, &size);
 		snprintf(sbuffer, sizeof(sbuffer) - 1,
 			 "Server: Accepting TCP 0:0:0:0:0:0:0:1 %d",
 			 ntohs(caddr.sin6_port));
 		set_enforce(1);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) | O_NONBLOCK);
 		show_prompt(sbuffer);
 		fd3 = accept(fd1, (struct sockaddr *) &caddr, &size);
-		show_result3(fd3);
+		show_result2(fd3);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) & ~O_NONBLOCK);
 
 		set_enforce(0);
 		close(fd2);
@@ -552,9 +545,11 @@ static void stage_network_test(void)
 			 "Server: Accepting TCP 0:0:0:0:0:0:0:1 %d",
 			 ntohs(caddr.sin6_port));
 		set_enforce(0);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) | O_NONBLOCK);
 		show_prompt(sbuffer);
 		fd3 = accept(fd1, (struct sockaddr *) &caddr, &size);
-		show_result3(fd3);
+		show_result2(fd3);
+		fcntl(fd1, F_SETFL, fcntl(fd1, F_GETFL, 0) & ~O_NONBLOCK);
 
 		close(fd3);
 		close(fd2);
@@ -579,11 +574,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(sbuffer);
 		show_result(bind(fd1, (struct sockaddr *) &saddr,
-				sizeof(saddr)));
+				 sizeof(saddr)));
 		getsockname(fd1, (struct sockaddr *) &saddr, &size);
 
 		fd2 = socket(PF_INET6, SOCK_DGRAM, 0);
@@ -594,11 +589,11 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(connect(fd2, (struct sockaddr *) &saddr,
-				   sizeof(saddr)));
+				    sizeof(saddr)));
 		getsockname(fd2, (struct sockaddr *) &caddr, &size);
 
 		snprintf(cbuffer, sizeof(cbuffer) - 1,
@@ -611,22 +606,22 @@ static void stage_network_test(void)
 		set_enforce(1);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
-				  sizeof(saddr)));
+				   sizeof(saddr)));
 		set_enforce(0);
 		show_prompt(cbuffer);
 		show_result(sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
-				  sizeof(saddr)));
+				   sizeof(saddr)));
 		set_enforce(1);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 		set_enforce(0);
 		sendto(fd2, "", 1, 0, (struct sockaddr *) &saddr,
 		       sizeof(saddr));
 		set_enforce(0);
 		show_prompt(sbuffer);
-		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, 0,
-				     (struct sockaddr *) &caddr, &size));
+		show_result2(recvfrom(fd1, buf, sizeof(buf) - 1, MSG_DONTWAIT,
+				      (struct sockaddr *) &caddr, &size));
 
 		close(fd2);
 		close(fd1);
@@ -647,5 +642,9 @@ int main(int argc, char *argv[])
 	}
 	stage_network_test();
 	clear_status();
+	if (0) { /* To suppress "defined but not used" warnings. */
+		write_domain_policy("", 0);
+		write_exception_policy("", 0);
+	}
 	return 0;
 }
