@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0+   2011/02/14
+ * Version: 1.8.1+   2011/05/11
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -131,12 +131,12 @@ static _Bool ccs_hexadecimal(const char c)
 }
 
 /**
- *  ccs_alphabet_char - Check whether the character is an alphabet.
+ * ccs_alphabet_char - Check whether the character is an alphabet.
  *
  * @c: The character to check.
  *
  * Returns true if @c is an alphabet character, false otherwise.
- */ 
+ */
 static _Bool ccs_alphabet_char(const char c)
 {
 	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z');
@@ -502,7 +502,7 @@ _Bool ccs_correct_domain(const char *domainname)
 		if (!cp)
 			break;
 		if (*domainname != '/' ||
-		    !ccs_correct_word2(domainname, cp - domainname - 1))
+		    !ccs_correct_word2(domainname, cp - domainname))
 			goto out;
 		domainname = cp + 1;
 	}
@@ -1155,7 +1155,7 @@ static int ccs_dump_index = 0;
  * @pid:   Pid to search.
  * @depth: Depth of the process for printing like pstree command.
  *
- * Returns nothing. 
+ * Returns nothing.
  */
 static void ccs_sort_process_entry(const pid_t pid, const int depth)
 {
@@ -1194,7 +1194,7 @@ static int ccs_task_entry_compare(const void *a, const void *b)
  *
  * @show_all: Ture if kernel threads should be included, false otherwise.
  *
- * Returns nothing. 
+ * Returns nothing.
  */
 void ccs_read_process_list(_Bool show_all)
 {
@@ -1435,7 +1435,7 @@ FILE *ccs_open_read(const char *filename)
  * @src:  Filename to save from.
  * @dest: Filename to save to.
  *
- * Returns true on success, false otherwise. 
+ * Returns true on success, false otherwise.
  */
 _Bool ccs_move_proc_to_file(const char *src, const char *dest)
 {
@@ -1514,7 +1514,7 @@ out:
  *
  * @dp: Pointer to "struct ccs_domain_policy".
  *
- * Returns nothing. 
+ * Returns nothing.
  */
 void ccs_clear_domain_policy(struct ccs_domain_policy *dp)
 {
@@ -1619,7 +1619,7 @@ static void ccs_sort_domain_policy(struct ccs_domain_policy *dp)
  * @dp:       Pointer to "struct ccs_domain_policy".
  * @filename: Domain policy's pathname.
  *
- * Returns nothing. 
+ * Returns nothing.
  */
 void ccs_read_domain_policy(struct ccs_domain_policy *dp, const char *filename)
 {
@@ -1703,7 +1703,7 @@ void ccs_delete_domain(struct ccs_domain_policy *dp, const int index)
  * @entry: String to add.
  * @index: Index in the @dp array.
  *
- * Returns 0 if successfully added or already exists, -EINVAL otherwise. 
+ * Returns 0 if successfully added or already exists, -EINVAL otherwise.
  */
 int ccs_add_string_entry(struct ccs_domain_policy *dp, const char *entry,
 			 const int index)
@@ -1750,7 +1750,7 @@ int ccs_add_string_entry(struct ccs_domain_policy *dp, const char *entry,
  * @index: Index in the @dp array.
  *
  * Returns 0 if successfully removed, -ENOENT if not found,
- * -EINVAL otherwise. 
+ * -EINVAL otherwise.
  */
 int ccs_del_string_entry(struct ccs_domain_policy *dp, const char *entry,
 			 const int index)
@@ -1801,7 +1801,7 @@ void ccs_handle_domain_policy(struct ccs_domain_policy *dp, FILE *fp,
 	if (!is_write)
 		goto read_policy;
 	while (true) {
-		char *line = ccs_freadline(fp);
+		char *line = ccs_freadline_unpack(fp);
 		_Bool is_delete = false;
 		_Bool is_select = false;
 		unsigned int profile;
@@ -1865,7 +1865,7 @@ static _Bool ccs_buffer_locked = false;
  * Returns nothing.
  *
  * This is for avoiding accidental overwriting.
- * ccs_freadline() and ccs_shprintf() have their own memory buffer. 
+ * ccs_freadline() and ccs_shprintf() have their own memory buffer.
  */
 void ccs_get(void)
 {
@@ -1880,7 +1880,7 @@ void ccs_get(void)
  * Returns nothing.
  *
  * This is for avoiding accidental overwriting.
- * ccs_freadline() and ccs_shprintf() have their own memory buffer. 
+ * ccs_freadline() and ccs_shprintf() have their own memory buffer.
  */
 void ccs_put(void)
 {
@@ -1890,18 +1890,16 @@ void ccs_put(void)
 }
 
 /**
- * ccs_shprintf - sprintf() to the shared buffer.
+ * ccs_shprintf - sprintf() to dynamically allocated buffer.
  *
  * @fmt: The printf()'s format string, followed by parameters.
  *
- * Returns pointer to the shared buffer.
+ * Returns pointer to dynamically allocated buffer.
  *
  * The caller must not free() the returned pointer.
  */
 char *ccs_shprintf(const char *fmt, ...)
 {
-	if (!ccs_buffer_locked)
-		ccs_out_of_memory();
 	while (true) {
 		static char *policy = NULL;
 		static int max_policy_len = 0;
@@ -1925,11 +1923,11 @@ char *ccs_shprintf(const char *fmt, ...)
 }
 
 /**
- * ccs_freadline - Read a line from file to the shared buffer.
+ * ccs_freadline - Read a line from file to dynamically allocated buffer.
  *
  * @fp: Pointer to "FILE".
  *
- * Returns pointer to the shared buffer on success, NULL otherwise.
+ * Returns pointer to dynamically allocated buffer on success, NULL otherwise.
  *
  * The caller must not free() the returned pointer.
  */
@@ -1937,8 +1935,6 @@ char *ccs_freadline(FILE *fp)
 {
 	static char *policy = NULL;
 	int pos = 0;
-	if (!ccs_buffer_locked)
-		ccs_out_of_memory();
 	while (true) {
 		static int max_policy_len = 0;
 		const int c = fgetc(fp);
@@ -1965,6 +1961,112 @@ char *ccs_freadline(FILE *fp)
 	return policy;
 }
 
+/**
+ * ccs_freadline_unpack - Read a line from file to dynamically allocated buffer.
+ *
+ * @fp: Pointer to "FILE". Maybe NULL.
+ *
+ * Returns pointer to dynamically allocated buffer on success, NULL otherwise.
+ *
+ * The caller must not free() the returned pointer.
+ *
+ * The caller must repeat calling this function without changing @fp (or with
+ * changing @fp to NULL) until this function returns NULL, for this function
+ * caches a line if the line is packed. Otherwise, some garbage lines might be
+ * returned to the caller.
+ */
+char *ccs_freadline_unpack(FILE *fp)
+{
+	static char *previous_line = NULL;
+	static char *cached_line = NULL;
+	static int pack_start = 0;
+	static int pack_len = 0;
+	if (cached_line)
+		goto unpack;
+	if (!fp)
+		return NULL;
+	{
+		char *pos;
+		unsigned int offset;
+		unsigned int len;
+		char *line = ccs_freadline(fp);
+		if (!line)
+			return NULL;
+		if (sscanf(line, "acl_group %u", &offset) == 1 && offset < 256)
+			pos = strchr(line + 11, ' ');
+		else
+			pos = NULL;
+		if (pos++)
+			offset = pos - line;
+		else
+			offset = 0;
+		if (!strncmp(line + offset, "file ", 5)) {
+			char *cp = line + offset + 5;
+			char *cp2 = strchr(cp + 1, ' ');
+			len = cp2 - cp;
+			if (cp2 && memchr(cp, '/', len)) {
+				pack_start = cp - line;
+				goto prepare;
+			}
+		} else if (!strncmp(line + offset, "network ", 8)) {
+			char *cp = strchr(line + offset + 8, ' ');
+			char *cp2 = NULL;
+			if (cp)
+				cp = strchr(cp + 1, ' ');
+			if (cp)
+				cp2 = strchr(cp + 1, ' ');
+			cp++;
+			len = cp2 - cp;
+			if (cp2 && memchr(cp, '/', len)) {
+				pack_start = cp - line;
+				goto prepare;
+			}
+		}
+		return line;
+prepare:
+		pack_len = len;
+		cached_line = strdup(line);
+		if (!cached_line)
+			ccs_out_of_memory();
+	}
+unpack:
+	{
+		char *line = NULL;
+		char *pos = cached_line + pack_start;
+		char *cp = memchr(pos, '/', pack_len);
+		unsigned int len = cp - pos;
+		free(previous_line);
+		previous_line = NULL;
+		if (!cp) {
+			previous_line = cached_line;
+			cached_line = NULL;
+			line = previous_line;
+		} else if (pack_len == 1) {
+			/* Ignore trailing empty word. */
+			free(cached_line);
+			cached_line = NULL;
+		} else {
+			/* Current string is "abc d/e/f ghi". */
+			line = strdup(cached_line);
+			if (!line)
+				ccs_out_of_memory();
+			previous_line = line;
+			/* Overwrite "abc d/e/f ghi" with "abc d ghi". */
+			memmove(line + pack_start + len, pos + pack_len,
+				strlen(pos + pack_len) + 1);
+			/* Overwrite "abc d/e/f ghi" with "abc e/f ghi". */
+			cp++;
+			memmove(pos, cp, strlen(cp) + 1);
+			/* Forget "d/" component. */
+			pack_len -= len + 1;
+			/* Ignore leading and middle empty word. */
+			if (!len)
+				goto unpack;
+		}
+		return line;
+	}
+}
+
 /**
  * ccs_check_remote_host - Check whether the remote host is running with the TOMOYO 1.8 kernel or not.
  *
